require 'rubygems'
require 'json'
require 'puppet'

class SimpleRest
    # instance variables
    @url      = ""
    @username = ""
    @password = ""
    
    # setup the REST object
    def initialize(url, username, password)
        @url = url

        # login credentials
        @username = username
        @password = password
    end

    def getUrl
        return "https://#{@url}/securechangeworkflow/api/secureapp/repository/puppet_api/connections.json"
    end
    
    
      def get(params)
          uri = getUrl()
          begin
              full_url =  uri +"?ip=" + params
              parsed_url = URI.parse(full_url)
              http_object = Net::HTTP::new(parsed_url.host, parsed_url.port)
              http_object.use_ssl = true
              http_object.ssl_version = :SSLv23
	      http_object.verify_mode= OpenSSL::SSL::VERIFY_NONE
              http_request = Net::HTTP::Get.new(parsed_url.request_uri)
              http_request.basic_auth(@username, @password)
              http_response = http_object.request(http_request)
              return JSON.parse(http_response.body)
          rescue Errno::ECONNREFUSED => e
              # connection error
              raise Puppet::ParseError, "tufin_secureapp: Unable to connect to #{req} - #{e.message}"
          rescue SocketError => e
              # socket error, maybe the URL can't be reached
              raise Puppet::ParseError, "tufin_secureapp: Unable to connect to #{req} - #{e.message}"
          rescue JSON::ParserError => e
              # bad JSON returned
              raise Puppet::ParseError, "tufin_secureapp: Unable to parse JSON - #{e.message}"
          rescue Exception => e
              # handle unknown exceptions
              raise Puppet::ParseError, "tufin_secureapp: #{e.message}"
          end
      end
end

    def mask_2_ciddr mask
      "/" + mask.split(".").map { |e| e.to_i.to_s(2).rjust(8, "0") }.join.count("1").to_s
    end

# expose the function to puppet
module Puppet::Parser::Functions
    newfunction(:tufin_secureapp, :type => :rvalue, :doc => <<-'ENDHEREDOC') do |args|
        Makes an API call to the Tufin SecureApp server and returns a collection of firewall rules if any.

        For example:

            $rules = tufin_secureapp(
                '192.168.1.80', 'JohnDoe', 'johnspassword',
                [ '192.168.1.76', '192.168.1.72' ]
            )

        If the API returns no firewall rules, an empty object is returned instead.

        ENDHEREDOC

        if args.length < 4
            raise Puppet::ParseError, "tufin_secureapp: wrong number of arguments (got #{args.length}, must be at 4)"
        end

        url      = args[0]
        username = args[1]
        password = args[2]
        ips      = args[3].join(',')

	ips.delete("127.0.0.1")

        # validate the required arguments
        unless url.is_a?(String)
            raise Puppet::ParseError, "tufin_secureapp: unexpected argument type #{url.class}, `url` should be a string"
        end

        unless username.is_a?(String)
            raise Puppet::ParseError, "tufin_secureapp: unexpected argument type #{username.class}, `username` should be a string"
        end

        unless password.is_a?(String)
            raise Puppet::ParseError, "tufin_secureapp: unexpected argument type #{password.class}, `password` should be a string"
        end

        unless ips.is_a?(Array) or ips.is_a?(String)
            raise Puppet::ParseError, "tufin_secureapp: unexpected argument type #{ips.class}, `ips` should be an array or string"
        end

        # if `ips` is an array, check that each element is a string
        if ips.is_a?(Array)
            ips.each do |ip|
                unless ip.is_a?(String)
                    raise Puppet::ParseError, "tufin_secureapp: unexpected argument type #{ip.class}, `#{ip}` should be a string"
                end
            end
        end

        print "URL " + url + "\n"

        # create REST object
        rest = SimpleRest.new(url, username, password)

        # get the hosts
        hosts = rest.get(ips)

        # collection of rules to send out
        rules = {}

        # firewall rule count
        count = 0

        # get the hosts object
        hosts = hosts["hosts"]["host"]

        # this may occasionally be a single object,
        # make sure it's always an array
        unless hosts.is_a?(Array)
            hosts = [ hosts ]
        end

        hosts.each do |host|
            # no connection, move along
            next if host["connections"] == ""

            connections = host["connections"]["connection"]

            # this may occasionally be a single object,
            # make sure it's always an array
            unless connections.is_a?(Array)
                connections = [ connections ]
            end

            # iterate the connections to generate firewall rules
            connections.each do |con|
                name         = con["name"]
                sources      = con["sources"]["source"]
                services     = con["services"]["service"]
                destinations = con["destinations"]["destination"]

                # generate a rule for each source - destination pair per service
                services.each do |service|
						proto = case service["type"]
							when "tcp_service" then "tcp"
							when "udp_service" then "udp"
							when "icmp_service" then "icmp"
							when "ip_service"
								case service["min"]
										when 6 then "tcp"
										when 17 then "udp"
										when 1 then "icmp"
										when 58 then "ipv6-icmp"
										when 50 then "esp"
										when 51 then "ah"
										when 112 then "vrrp"
										when 2 then "igmp"
										when 4 then "ipencap"
										when 89 then "ospf"
										when 47 then "gre"
										when 0
											if service["max"] == 255 then
												"all"
											end	
								end
							end
                    
					if service["type"] == "tcp_service" or service["type"] == "udp_service" then 					
						port = service["min"].to_s
				
						if service["max"] != service["min"] then
							port += "-" + service["max"].to_s
						end
					end
                    
					if proto.to_s.strip.length > 0 then 
						sources.each do |source|
							destinations.each do |dest|
								thisCount = "%03d" % count
								count += 1

								rule = {
									"action"      => "accept",
									"proto"       => proto,
									"dport"       => port
								}

								if source["type"] == "host" then
									rule["source"] = source["ip"]
								end

								if source["type"] == "subnet" then
									mask = mask_2_ciddr source["netmask"]
									rule["source"] = source["ip"].to_s + mask 
								end

								if source["type"] == "range" then
									rule["src_range"] = source["first_ip"].to_s + "-" + source["last_ip"].to_s
								end

								if dest["type"] == "host" then
									rule["destination"] = dest["ip"]
								end

								if dest["type"] == "subnet" then
									mask = mask_2_ciddr dest["netmask"]
									rule["destination"] = dest["ip"].to_s + mask
								end

								if dest["type"] == "range" then
									rule["dst_range"] = dest["first_ip"].to_s + "-" + dest["last_ip"].to_s
								end

								rules["#{thisCount} #{name}"] = rule

								# clear keys with empty values
								rules.each do |id, rule|
									rule.delete_if do |k, v|
										!v
									end
								end
							end
						end
					end
                end
            end
        end
        return rules
    end
end



